local revision = tonumber(("$Rev: 170 $"):match("(%d+)") or 1)

if CogsUtils and CogsUtils.revision >= revision then
	return
end

CogsUtils = { revision = revision }
local lib = CogsUtils

--[[ MakeCache makes a cache ]]
function lib:MakeCache(tbl, metamethod)
	if not metamethod then
		metamethod = tbl
		tbl = nil
	end
	return setmetatable(tbl or {}, {
		__index = function(tbl, key)
			local val = metamethod(key)
			tbl[key] = val
			return val
		end
	})
end



--[[ GetFuncify makes a ____ify function ]]
local funcifyFuncs = lib:MakeCache(function(func)
	local function funcify(...)
		if select("#", ...) > 0 then
			return func((...)), funcify(select(2, ...))
		end
	end
	return funcify
end)

function CogsUtils:GetFuncify(func)
	return funcifyFuncs[func]
end



--[[ GetPrint makes a print function ]]
local stringify = lib:GetFuncify(tostring)

local printFuncs = lib:MakeCache(function(header)
	if header ~= "" then
		return function(...)
			local first = select("#", ...) == 0 and "" or tostring(...)
			DEFAULT_CHAT_FRAME:AddMessage(strjoin(", ",
				header..tostring(first), stringify(select(2, ...))))
		end
	else
		return function(...)
			DEFAULT_CHAT_FRAME:AddMessage(strjoin(", ", stringify(...)))
		end
	end
end)

function lib:GetPrint(header)
	header = header or ""
	return printFuncs[header]
end
